#########################################################################################
###### Import / Export
#########################################################################################

#########################################################################################
#### Import / Export Settings execute
#########################################################################################

POPSaveSettings <- c()
loading <- FALSE

####
#### Function: Save Settings
############################
FSaveSettings <- function(SettingsFile) {
	FCheckDataInput(CheckInputDataInput)
	FFileInputCollect()
	FCheckLabelColour(CheckInputLabelColour)
	Sys.sleep(.2)
	FCheckGraphOpt(CheckInputGraphOpt)
	Sys.sleep(.2)
	FCheckAnalysisOpt(CheckInputAnalysisOpt)
	## check Graph output
	Sys.sleep(.2)
	FCheckDataOutput(CheckInputDataOutput)
	
	#### check status messages
	errormessages <- BXErrormessages$getChildren()
	proceed <- 1
	for (i in 1:length(errormessages)) {
		if (errormessages[[i]]["visible"] == TRUE) { #if any of the errormessages is visible produce popup
			POPErrorWarning <- gtkMessageDialog(POPSaveSettings, "destroy-with-parent", type = "question", buttons = "none", "There are invalid settings. Are you sure you want to save them?")
			POPErrorWarning$addButton("Save", 1)
			POPErrorWarning$addButton("Cancel", 2)
			initiation <- 1
			while (initiation == 1) {
				response <- POPErrorWarning$run()[1]
				if (response == 1) {
					proceed <- 1 #user actually wants to save invalid settings
					POPErrorWarning$destroy()
					initiation <- 0
				} else {
					proceed <- 0 #user wants to change settings before saving
					POPErrorWarning$destroy()
					initiation <- 0
				}
			}
			break
		}
	}
	
	if (proceed == 1 & FCheckFileExistance(SettingsFile) == TRUE) {
		PresentSettings <- list(	"DataLayout" = SETDataLayout, 
									"Borders" = SETBorders, 
									"FilesInput" = SETFilesInput,
									"LabelColour" = SETLabelNumberColour,
									"GraphOpt" = SETGraphOpt,
									"AnalysisOpt" = SETAnalysisOpt,
									"GraphOutput" = SETGraphOutput,
									"DataOutput" = SETDataOutput )
		#print(PresentSettings)
		SaveFile <- file(paste(getwd(), "/", SettingsFile, sep=""))
		open(SaveFile, open="w")
		serialize(PresentSettings, SaveFile)
		close(SaveFile)
		print(paste("Settings were saved to ", getwd(), "/", SettingsFile, sep=""))
	}
	
	if (class(POPSaveSettings)[[1]] == "GtkMessageDialog") {
		POPSaveSettings$destroy()
	}
}

####
#### Function: Load Settings
############################
FLoadSettings <- function(SettingsFile) {
	LoadFile <- file(SettingsFile)
	open(LoadFile, open="r")
	LoadedSettings <- unserialize(LoadFile)
	close(LoadFile)
	
	#print(LoadedSettings)
	
	## Clear Data sets
	Data_input <<- data.frame("empty" = "empty")
	Template <<- data.frame("empty" = "empty")
	TemplateAverage <<- data.frame("empty" = "empty")
	
	used <- SETDataLayout$Version
	usedNum <- unlist(strsplit(used, "\\."))
	loaded <- LoadedSettings[["DataLayout"]]$Version
	if (length(loaded) == 0) loaded <- "3 or lower"
	else loadedNum <- unlist(strsplit(loaded, "\\."))
	if (loaded == "3 or lower") FVersionTooLow(used, loaded)
	else if (loadedNum[1] < usedNum[1]) FVersionTooLow(used, loaded)
	else if (loadedNum[1] > usedNum[1]) FVersionTooHigh(used, loaded)
	else if (loadedNum[1] == usedNum[1] & loadedNum[3] != usedNum[3]) {
		if (usedNum[3] == "full" | loadedNum[3] == "base") FVersionTooLow(used, loaded)
		else if (loadedNum[3] == "full" | usedNum[3] == "base") FVersionTooHigh(used, loaded)
		else FVersionsDiffer(used, loaded)
	} else if (used != loaded) FVersionsDiffer(used, loaded)
	
	loading <<- TRUE
	FTranslateSettingsLabelColour(LoadedSettings[["LabelColour"]])
	loading <<- TRUE
	FTranslateSettingsDataInput(LoadedSettings[c("DataLayout", "Borders", "FilesInput")])
	loading <<- TRUE
	FTranslateSettingsGraphOpt(LoadedSettings[["GraphOpt"]])
	loading <<- TRUE
	FTranslateSettingsAnalysisOpt(LoadedSettings[["AnalysisOpt"]])
	loading <<- TRUE
	FTranslateSettingsGraphOutput(LoadedSettings[["GraphOutput"]])
	loading <<- TRUE
	FTranslateSettingsDataOutput(LoadedSettings[["DataOutput"]])
	if (class(POPSaveSettings)[[1]] == "GtkMessageDialog") {
		POPSaveSettings$destroy()
	}
}

####
#### Function: Notify Version too high
######################################
FVersionTooHigh <- function(used, loaded) {
	POPVersionTooHigh <- gtkMessageDialog(mainApplication, "destroy-with-parent", type="error", button ="ok", paste0("The settings you are trying to load are from TReCCA version ", loaded, ", but you are only using TReCCA version ", used, ". You won't be able to use part of the settings you are trying to load or to reproduce the analysis. If you want to do so, please download a higher version of the program."))
	POPVersionTooHigh$run()
	POPVersionTooHigh$destroy()
}
####
#### Function: Notify Version too low
#####################################
FVersionTooLow <- function(used, loaded) {
	POPVersionTooLow <- gtkMessageDialog(mainApplication, "destroy-with-parent", type="error", button ="ok", paste0("The settings you are trying to load are from TReCCA version ", loaded, ", but you are using the more powerful TReCCA version ", used, ". The missing settings are replaced by default values."))
	POPVersionTooLow$run()
	POPVersionTooLow$destroy()
}
####
#### Function: Notify Versions differ
#####################################
FVersionsDiffer <- function(used, loaded) {
	POPVersionsDiffer <- gtkMessageDialog(mainApplication, "destroy-with-parent", type="error", button ="ok", paste0("The settings you are trying to load are from TReCCA version ", loaded, ", but you are using TReCCA version ", used, ". Since the settings or the computational design of these two versions differ, you might not be able to reproduce certain analyses and you might encounter default settings instead of your personalised ones."))
	POPVersionsDiffer$run()
	POPVersionsDiffer$destroy()
}
####
#### Function: Save RData
#########################
FSaveRData <- function(RDataFile) {
	if (FCheckFileExistance(RDataFile) == TRUE) {
			
		SaveList <- c("TEMPChecksums", "Template", "TemplateAverage")
		for (i in 1:length(ListDataSets)) {
			SaveList <- c(SaveList, names(ListDataSets[i]))
		}
		
		SaveFile <- file(paste(getwd(), "/", RDataFile, sep=""))
		open(SaveFile, open="wb")
		save(list = SaveList, file = SaveFile)
		close(SaveFile)
		print(paste("R-Data was saved to ", getwd(), "/", RDataFile, sep=""))
	}
	
	if (class(POPSaveRData)[[1]] == "GtkMessageDialog") {
		POPSaveRData$destroy()
	}
}

#########################################################################################
#### Import / Export Settings execute bottom
#########################################################################################

####
#### Function: Setting -> Widgetfunction Data Input
###################################################
FTranslateSettingsDataInput <- function(Loaded) {
	#### Translate all Data Input Settings to Widget Actions:
	
	#### Header Settings
	# Header exists?
	if (length(Loaded[[2]]$header) != 0 & Loaded[[2]]$header == FALSE) btnBordersHeader1N$setActive(is.active = TRUE)
	else btnBordersHeader1Y$setActive(is.active = TRUE)
	# rownumber Header?
	ENTHeader$setText(switch(class(Loaded[[2]]$rowsTop), NULL = "NA", numeric = Loaded[[2]]$rowsTop))
	# remove Header?
	if (length(Loaded[[2]]$remove) != 0 & Loaded[[2]]$remove == TRUE) btnBordersHeader2Y$setActive(is.active = TRUE)
	else btnBordersHeader2N$setActive(is.active = TRUE)
	
	#### Data Input Settings
	# Single document?
	if (length(Loaded[[1]]$multiple) != 0 & Loaded[[1]]$multiple == TRUE) btnDataLayoutMultipleY$setActive(is.active = TRUE)
	else btnDataLayoutMultipleN$setActive(is.active = TRUE)
	# columns to be deleted
	ENTremoveStart$setText(switch(class(Loaded[[2]]$removeStart), NULL = "NA", numeric = Loaded[[2]]$removeStart))
	ENTremoveEnd$setText(switch(class(Loaded[[2]]$removeEnd), NULL = "NA", numeric = Loaded[[2]]$removeEnd))
	# rows to be deleted
	ENTRowsEnd$setText(switch(class(Loaded[[2]]$rowsEnd), NULL = "NA", numeric = Loaded[[2]]$rowsEnd))
	# platenumber
	ENTDataLayoutPlate$setText(switch(class(Loaded[[1]]$platenumber), NULL = "NA", numeric = Loaded[[1]]$platenumber))
	# Data cleaned?
	if (length(Loaded[[1]]$missingValues) != 0 & Loaded[[1]]$missingValues == TRUE) btnDataLayoutNumericalN$setActive(is.active = TRUE)
	else btnDataLayoutNumericalY$setActive(is.active = TRUE)
	# Timescale input?
	if (length(Loaded[[1]]$timescale) != 0) {
		if (Loaded[[1]]$timescale == 1) btnDataLayoutTimesec$setActive(is.active = TRUE)
		else if (Loaded[[1]]$timescale == 60) btnDataLayoutTimemin$setActive(is.active = TRUE)
		else if (Loaded[[1]]$timescale == 3600) btnDataLayoutTimeh$setActive(is.active = TRUE)
		else if (Loaded[[1]]$timescale == 86400) btnDataLayoutTimeday$setActive(is.active = TRUE)
	}
	if (SETDataLayout$Version != "4.0.base" & length(Loaded[[1]]$inputunit) != 0) {
		# Unit input?
		if (Loaded[[1]]$inputunit == "OD" & SETDataLayout$Version != "4.0.96well") btnDataLayoutUnitOxoDish$setActive(is.active = TRUE)
		else if (Loaded[[1]]$inputunit == "OP" & SETDataLayout$Version != "4.0.24well") btnDataLayoutUnitOxoPlate$setActive(is.active = TRUE)
		else if (Loaded[[1]]$inputunit == "HP" & SETDataLayout$Version != "4.0.24well") btnDataLayoutUnitHydroPlate$setActive(is.active = TRUE)
		else if (Loaded[[1]]$inputunit == "none") btnDataLayoutUnitElse$setActive(is.active = TRUE)
		gSignalEmit(btnDataLayoutUnitElse, "toggled")
	}
	
	#### Files Input Settings
	# separator
	ENTInputSeparator$setText(switch(class(Loaded[[3]]$separator), NULL = "NA", character = Loaded[[3]]$separator))
	ENTInputDecimal$setText(switch(class(Loaded[[3]]$decimal), NULL = "NA", character = Loaded[[3]]$decimal))
	# filenames
	if (length(Loaded[[1]]$multiple) != 0 & Loaded[[1]]$multiple == TRUE) {
		for (i in 1:Loaded[[1]]$platenumber) {
		VARFileInputVectorPaths[[i]]$setText(Loaded[[3]]$files[i])
		}
	} else {
		ENTFileInputPath1$setText(switch(class(Loaded[[3]]$files[1]), NULL = "missing", character = Loaded[[3]]$files[1]))
	}
	# wellnumbers
	for (i in 1:Loaded[[1]]$platenumber) {
		VARFileInputVectorWells[[i]]$setText(Loaded[[3]]$wells[i])
	}
	
	#### Evaluate all buttons to change file list appearance
	loading <<- FALSE
	FCheckDataInput(CheckInputDataInput)
	
	#### Put filenames to settings:#
	FFileInputCollect()
}

####
#### Function: Setting -> Widgetfunction Labels & colours
#########################################################
FTranslateSettingsLabelColour <- function(Loaded) {
	# Template path
	ENTTemplatePath$setText(switch(class(Loaded$templatePath), NULL = "", character = Loaded$templatePath))
	# Template separator
	ENTTemplateSeparator$setText(switch(class(Loaded$templateSep), NULL = "NA", character = Loaded$templateSep))
	
	#### Evaluate all fields
	loading <<- FALSE
	FCheckLabelColour(CheckInputLabelColour)
	
	#### Load Template
	if (file.exists(SETLabelNumberColour$templatePath) == TRUE) FLoadTemplate(SETLabelNumberColour$templatePath, SETLabelNumberColour$templateSep)
}

####
#### Function: Setting -> Widgetfunction Graph Options
######################################################
FTranslateSettingsGraphOpt <- function(Loaded) {
	#### Titles and Subtitles
	# Title
	ENTPlotTitle$setText(switch(class(Loaded$Title), NULL = "missing", character = Loaded$Title))
	CKBPlotTitle$setActive(switch(class(Loaded$TitleShow), NULL = TRUE, logical = Loaded$TitleShow))
	CKBSubs$setActive(switch(class(Loaded$SubsShow), NULL = TRUE, logical = Loaded$SubsShow))
	# Subtitles
	ENTSubRaw$setText(switch(class(Loaded$Subtitles["raw"]), NULL = "missing", character = Loaded$Subtitles["raw"], list = Loaded$Subtitles[["raw"]]))
	ENTSubRawSm$setText(switch(class(Loaded$Subtitles["rawSm"]), NULL = "missing", character = Loaded$Subtitles["rawSm"], list = Loaded$Subtitles[["rawSm"]]))
	ENTSubRawSlope$setText(switch(class(Loaded$Subtitles["rawSlope"]), NULL = "missing", character = Loaded$Subtitles["rawSlope"], list = Loaded$Subtitles[["rawSlope"]]))
	ENTSubRawCons$setText(switch(class(Loaded$Subtitles["rawCons"]), NULL = "missing", character = Loaded$Subtitles["rawCons"], list = "missing"))
	ENTSubRawAv$setText(switch(class(Loaded$Subtitles["rawAv"]), NULL = "missing", character = Loaded$Subtitles["rawAv"], list = Loaded$Subtitles[["rawAv"]]))
	ENTSubRawAvSm$setText(switch(class(Loaded$Subtitles["rawAvSm"]), NULL = "missing", character = Loaded$Subtitles["rawAvSm"], list = Loaded$Subtitles[["rawAvSm"]]))
	ENTSubRawAvSlope$setText(switch(class(Loaded$Subtitles["rawAvSlope"]), NULL = "missing", character = Loaded$Subtitles["rawAvSlope"], list = Loaded$Subtitles[["rawAvSlope"]]))
	ENTSubRawAvCons$setText(switch(class(Loaded$Subtitles["rawAvCons"]), NULL = "missing", character = Loaded$Subtitles["rawAvCons"], list = "missing"))
	if (SETDataLayout$Version == "4.0.full" | SETDataLayout$Version == "4.0.24well") {
		ENTSubOxoDish$setText(switch(class(Loaded$Subtitles["oxoDish"]), NULL = "missing", character = Loaded$Subtitles["oxoDish"], list = Loaded$Subtitles[["oxoDish"]]))
		ENTSubOxoDishSm$setText(switch(class(Loaded$Subtitles["oxoDishSm"]), NULL = "missing", character = Loaded$Subtitles["oxoDishSm"], list = Loaded$Subtitles[["oxoDishSm"]]))
		ENTSubOxoDishSlope$setText(switch(class(Loaded$Subtitles["oxoDishSlope"]), NULL = "missing", character = Loaded$Subtitles["oxoDishSlope"], list = Loaded$Subtitles[["oxoDishSlope"]]))
		ENTSubOxoDishCons$setText(switch(class(Loaded$Subtitles["oxoDishCons"]), NULL = "missing", character = Loaded$Subtitles["oxoDishCons"], list = "missing"))
		ENTSubOxoDishAv$setText(switch(class(Loaded$Subtitles["oxoDishAv"]), NULL = "missing", character = Loaded$Subtitles["oxoDishAv"], list = Loaded$Subtitles[["oxoDishAv"]]))
		ENTSubOxoDishAvSm$setText(switch(class(Loaded$Subtitles["oxoDishAvSm"]), NULL = "missing", character = Loaded$Subtitles["oxoDishAvSm"], list = Loaded$Subtitles[["oxoDishAvSm"]]))
		ENTSubOxoDishAvSlope$setText(switch(class(Loaded$Subtitles["oxoDishAvSlope"]), NULL = "missing", character = Loaded$Subtitles["oxoDishAvSlope"], list = Loaded$Subtitles[["oxoDishAvSlope"]]))
		ENTSubOxoDishAvCons$setText(switch(class(Loaded$Subtitles["oxoDishAvCons"]), NULL = "missing", character = Loaded$Subtitles["oxoDishAvCons"], list = "missing"))
	}
	if (SETDataLayout$Version == "4.0.full" | SETDataLayout$Version == "4.0.96well") {
		ENTSubOxoPlate$setText(switch(class(Loaded$Subtitles["oxoPlate"]), NULL = "missing", character = Loaded$Subtitles["oxoPlate"], list = Loaded$Subtitles[["oxoPlate"]]))
		ENTSubOxoPlateSm$setText(switch(class(Loaded$Subtitles["oxoPlateSm"]), NULL = "missing", character = Loaded$Subtitles["oxoPlateSm"], list = Loaded$Subtitles[["oxoPlateSm"]]))
		ENTSubOxoPlateSlope$setText(switch(class(Loaded$Subtitles["oxoPlateSlope"]), NULL = "missing", character = Loaded$Subtitles["oxoPlateSlope"], list = Loaded$Subtitles[["oxoPlateSlope"]]))
		ENTSubOxoPlateCons$setText(switch(class(Loaded$Subtitles["oxoPlateCons"]), NULL = "missing", character = Loaded$Subtitles["oxoPlateCons"], list = "missing"))
		ENTSubOxoPlateAv$setText(switch(class(Loaded$Subtitles["oxoPlateAv"]), NULL = "missing", character = Loaded$Subtitles["oxoPlateAv"], list = Loaded$Subtitles[["oxoPlateAv"]]))
		ENTSubOxoPlateAvSm$setText(switch(class(Loaded$Subtitles["oxoPlateAvSm"]), NULL = "missing", character = Loaded$Subtitles["oxoPlateAvSm"], list = Loaded$Subtitles[["oxoPlateAvSm"]]))
		ENTSubOxoPlateAvSlope$setText(switch(class(Loaded$Subtitles["oxoPlateAvSlope"]), NULL = "missing", character = Loaded$Subtitles["oxoPlateAvSlope"], list = Loaded$Subtitles[["oxoPlateAvSlope"]]))
		ENTSubOxoPlateAvCons$setText(switch(class(Loaded$Subtitles["oxoPlateAvCons"]), NULL = "missing", character = Loaded$Subtitles["oxoPlateAvCons"], list = "missing"))
		
		ENTSubHydroPlate$setText(switch(class(Loaded$Subtitles["hydroPlate"]), NULL = "missing", character = Loaded$Subtitles["hydroPlate"], list = Loaded$Subtitles[["hydroPlate"]]))
		ENTSubHydroPlateCal$setText(switch(class(Loaded$Subtitles["hydroPlateCal"]), NULL = "missing", character = Loaded$Subtitles["hydroPlateCal"], list = Loaded$Subtitles[["hydroPlateCal"]]))
		ENTSubHydroPlateSm$setText(switch(class(Loaded$Subtitles["hydroPlateSm"]), NULL = "missing", character = Loaded$Subtitles["hydroPlateSm"], list = Loaded$Subtitles[["hydroPlateSm"]]))
		ENTSubHydroPlateSlope$setText(switch(class(Loaded$Subtitles["hydroPlateSlope"]), NULL = "missing", character = Loaded$Subtitles["hydroPlateSlope"], list = Loaded$Subtitles[["hydroPlateSlope"]]))
		ENTSubHydroPlateAv$setText(switch(class(Loaded$Subtitles["hydroPlateAv"]), NULL = "missing", character = Loaded$Subtitles["hydroPlateAv"], list = Loaded$Subtitles[["hydroPlateAv"]]))
		ENTSubHydroPlateAvSm$setText(switch(class(Loaded$Subtitles["hydroPlateAvSm"]), NULL = "missing", character = Loaded$Subtitles["hydroPlateAvSm"], list = Loaded$Subtitles[["hydroPlateAvSm"]]))
		ENTSubHydroPlateAvSlope$setText(switch(class(Loaded$Subtitles["hydroPlateAvSlope"]), NULL = "missing", character = Loaded$Subtitles["hydroPlateAvSlope"], list = Loaded$Subtitles[["hydroPlateAvSlope"]]))
	}
	ENTSubMedium$setText(switch(class(Loaded$Subtitles["medium"]), NULL = "missing", character = Loaded$Subtitles["medium"], list = Loaded$Subtitles[["medium"]]))
	ENTSubMediumSm$setText(switch(class(Loaded$Subtitles["mediumSm"]), NULL = "missing", character = Loaded$Subtitles["mediumSm"], list = Loaded$Subtitles[["mediumSm"]]))
	ENTSubMediumSlope$setText(switch(class(Loaded$Subtitles["mediumSlope"]), NULL = "missing", character = Loaded$Subtitles["mediumSlope"], list = Loaded$Subtitles[["mediumSlope"]]))
	ENTSubMediumCons$setText(switch(class(Loaded$Subtitles["mediumCons"]), NULL = "missing", character = Loaded$Subtitles["mediumCons"], list = "missing"))
	ENTSubMediumAv$setText(switch(class(Loaded$Subtitles["mediumAv"]), NULL = "missing", character = Loaded$Subtitles["mediumAv"], list = Loaded$Subtitles[["mediumAv"]]))
	ENTSubMediumAvSm$setText(switch(class(Loaded$Subtitles["mediumAvSm"]), NULL = "missing", character = Loaded$Subtitles["mediumAvSm"], list = Loaded$Subtitles[["mediumAvSm"]]))
	ENTSubMediumAvSlope$setText(switch(class(Loaded$Subtitles["mediumAvSlope"]), NULL = "missing", character = Loaded$Subtitles["mediumAvSlope"], list = Loaded$Subtitles[["mediumAvSlope"]]))
	ENTSubMediumAvCons$setText(switch(class(Loaded$Subtitles["mediumAvCons"]), NULL = "missing", character = Loaded$Subtitles["mediumAvCons"], list = "missing"))
	ENTSubIC50Time$setText(switch(class(Loaded$Subtitles["IC50Time"]), NULL = "missing", character = Loaded$Subtitles["IC50Time"], list = Loaded$Subtitles[["IC50Time"]]))
	ENTSubIC50Fits$setText(switch(class(Loaded$Subtitles["IC50Fits"]), NULL = "missing", character = Loaded$Subtitles["IC50Fits"], list = Loaded$Subtitles[["IC50Fits"]]))
	
	#### Axes settings
	# X Scale
	if (length(Loaded$XScale) != 0) {
		if (Loaded$XScale == 1) btnPlotAxesH1sec$setActive(is.active = TRUE)
		else if (Loaded$XScale == 60) btnPlotAxesH1min$setActive(is.active = TRUE)
		else if (Loaded$XScale == 3600) btnPlotAxesH1h$setActive(is.active = TRUE)
		else if (Loaded$XScale == 86400) btnPlotAxesH1day$setActive(is.active = TRUE)
	}
	gSignalEmit(btnPlotAxesH1sec, "toggled") #activate one of them before importing the label setting
	# X Label
	ENTPlotAxesH2$setText(switch(class(Loaded$XLabel), NULL = "missing", character = Loaded$XLabel)) #replace automatic label with saved one
	# X Limits
	ENTPlotAxesH3min$setText(switch(class(Loaded$XLimits[1]), NULL = "missing", numeric = Loaded$XLimits[1]))
	ENTPlotAxesH3max$setText(switch(class(Loaded$XLimits[2]), NULL = "missing", numeric = Loaded$XLimits[2]))
	# Y Label
	ENTPlotAxesH5$setText(switch(class(Loaded$YLabel), NULL = "missing", character = Loaded$YLabel)) #replace automatic label with saved one
	# Y Limits
	ENTPlotAxesH6min$setText(switch(class(Loaded$YLimits[1]), NULL = "missing", numeric = Loaded$YLimits[1]))
	ENTPlotAxesH6max$setText(switch(class(Loaded$YLimits[2]), NULL = "missing", numeric = Loaded$YLimits[2]))
	
	#### Check all fields again to set Settings
	loading <<- FALSE
	FCheckGraphOpt(CheckInputGraphOpt)
}

####
#### Function: Setting -> Widgetfunction Analysis Options
#########################################################
FTranslateSettingsAnalysisOpt <- function(Loaded) {
	#### Selections
	CKBSelectionRawAv$setActive(is.active = as.logical(Loaded$raw[2]))
	CKBSelectionMedium$setActive(is.active = as.logical(Loaded$medium[1]))
	CKBSelectionMediumAv$setActive(is.active = as.logical(Loaded$medium[2]))
	CKBSelectionSmooth$setActive(is.active = as.logical(Loaded$smoothParam["calc"]))
	CKBSelectionSlope$setActive(is.active = as.logical(Loaded$slopeParam[["calc"]]))
	CKBSelectionCons$setActive(is.active = as.logical(Loaded$ConsParam["calc"]))
	CKBSelectionIC50$setActive(is.active = as.logical(Loaded$IC50calc))
	
	#### Formatting
	ENTFormatCutXStart$setText(switch(class(Loaded$FormattingParams["t1"]), NULL = "NA", numeric = Loaded$FormattingParams["t1"]))
	ENTFormatCutXEnd$setText(switch(class(Loaded$FormattingParams["t2"]), NULL = "NA", numeric = Loaded$FormattingParams["t2"]))
	ENTFormatTransformXm$setText(switch(class(Loaded$FormattingParams["m"]), NULL = "NA", numeric = Loaded$FormattingParams["m"]))
	ENTFormatTransformXb$setText(switch(class(Loaded$FormattingParams["b"]), NULL = "NA", numeric = Loaded$FormattingParams["b"]))
	ENTFormatTransformY$setText(switch(class(Loaded$FormattingParams["y"]), NULL = "NA", numeric = Loaded$FormattingParams["y"]))
	
	#### OxoDish
	if (SETDataLayout$Version == "4.0.full" | SETDataLayout$Version == "4.0.24well") {
		CKBSelectionOxoDish$setActive(is.active = as.logical(Loaded$oxoDish[1]))
		CKBSelectionOxoDishAv$setActive(is.active = as.logical(Loaded$oxoDish[2]))
		ENTOxoDishCalNumber$setText(switch(class(Loaded$range[1]), NULL = "NA", numeric = Loaded$range[1]))
		ENTOxoDishCalLast$setText(switch(class(Loaded$range[2]), NULL = "NA", numeric = Loaded$range[2]))
		ENTOxoDishCalTarget$setText(switch(class(Loaded$oxoDishTarget), NULL = "NA", numeric = Loaded$oxoDishTarget))
		ENTOxoDishCalSpan$setText(switch(class(Loaded$span), NULL = "NA", numeric = Loaded$span))
		ENTOxoDishCalPlateau$setText(switch(class(Loaded$plateau), NULL = "NA", numeric = Loaded$plateau))
		CKBSmOxoDish$setActive(is.active = as.logical(Loaded$smoothSelection[["Data_oxoDish"]]))
		CKBSmOxoDishAv$setActive(is.active = as.logical(Loaded$smoothSelection[["Data_oxoDishAv"]]))
		CKBSlopeOxoDish$setActive(is.active = as.logical(Loaded$slopeSelection[["Data_oxoDish"]]))
		CKBSlopeOxoDishAv$setActive(is.active = as.logical(Loaded$slopeSelection[["Data_oxoDishAv"]]))
		CKBConsOxoDish$setActive(is.active = as.logical(Loaded$ConsSelection[["Data_oxoDish"]]))
		CKBConsOxoDishAv$setActive(is.active = as.logical(Loaded$ConsSelection[["Data_oxoDishAv"]]))
	}
	
	#### OxoPlates
	if (SETDataLayout$Version == "4.0.full" | SETDataLayout$Version == "4.0.96well") {
		CKBSelectionOxoPlate$setActive(is.active = as.logical(Loaded$oxoPlate[1]))
		CKBSelectionOxoPlateAv$setActive(is.active = as.logical(Loaded$oxoPlate[2]))
		ENTOxoPlateCal0$setText(switch(class(Loaded$cal0100[1]), NULL = "missing", character = Loaded$cal0100[1]))
		ENTOxoPlateCal100$setText(switch(class(Loaded$cal0100[2]), NULL = "missing", character = Loaded$cal0100[2]))
		COMOxoPlateCalTargetUnit$setActive(switch(class(Loaded$unitselections[1]), NULL = 0, numeric = Loaded$unitselections[1]))
		COMOxoPlateCalTPres$setActive(switch(class(Loaded$unitselections[2]), NULL = 0, numeric = Loaded$unitselections[2]))
		COMOxoPlateCalTTemp$setActive(switch(class(Loaded$unitselections[3]), NULL = 0, numeric = Loaded$unitselections[3]))
		if (ENTOxoPlateCalTfactor["editable"] == TRUE) ENTOxoPlateCalTfactor$setText(switch(class(Loaded$factor), NULL = "NA", numeric = Loaded$factor))
		CKBSmOxoPlate$setActive(is.active = as.logical(Loaded$smoothSelection[["Data_oxoPlate"]]))
		CKBSmOxoPlateAv$setActive(is.active = as.logical(Loaded$smoothSelection[["Data_oxoPlateAv"]]))
		CKBSlopeOxoPlate$setActive(is.active = as.logical(Loaded$slopeSelection[["Data_oxoPlate"]]))
		CKBSlopeOxoPlateAv$setActive(is.active = as.logical(Loaded$slopeSelection[["Data_oxoPlateAv"]]))
		CKBConsOxoPlate$setActive(is.active = as.logical(Loaded$ConsSelection[["Data_oxoPlate"]]))
		CKBConsOxoPlateAv$setActive(is.active = as.logical(Loaded$ConsSelection[["Data_oxoPlateAv"]]))
		
		CKBSelectionHydroPlate$setActive(is.active = as.logical(Loaded$hydroPlate[1]))
		CKBSelectionHydroPlateAv$setActive(is.active = as.logical(Loaded$hydroPlate[2]))
		CKBSmHydroPlate$setActive(is.active = as.logical(Loaded$smoothSelection["Data_hydroPlate"]))
		CKBSmHydroPlateAv$setActive(is.active = as.logical(Loaded$smoothSelection["Data_hydroPlateAv"]))
		CKBSlopeHydroPlate$setActive(is.active = as.logical(Loaded$slopeSelection["Data_hydroPlate"]))
		CKBSlopeHydroPlateAv$setActive(is.active = as.logical(Loaded$slopeSelection["Data_hydroPlateAv"]))
		if (length(Template$Name) != 0 & length(Loaded$hydroPlateCal$pH) != 0) {
			for (i in 1:length(Template$Name)) {
				gtkEntrySetText(TEMPpHs[[Template$Name[i]]]$entry, Loaded$hydroPlateCal$pH[which(Loaded$hydroPlateCal$Name == Template$Name[i])])
			}
		}
	}
	
	#### Medium
	ENTMediumCalName$setText(switch(class(Loaded$name), NULL = "missing", character = Loaded$name))
	ENTMediumCalTarget$setText(switch(class(Loaded$medTarget), NULL = "NA", numeric = Loaded$medTarget))
	
	#### Average
	if (length(Loaded$average) != 0) {
		if (Loaded$average == "name") btnAverageName$setActive(is.active = TRUE)
		else if (Loaded$average == "number") btnAverageNumber$setActive(is.active = TRUE)
		else if (Loaded$average == "colour") btnAverageColour$setActive(is.active = TRUE)
	}
	CKBAverageSD$setActive(is.active = as.logical(Loaded$SD))
	
	#### Smoothing
	ENTSmoothOption$setText(switch(class(Loaded$smoothParam[2]), NULL = "NA", numeric = Loaded$smoothParam[2]))
	CKBSmRaw$setActive(is.active = as.logical(Loaded$smoothSelection[["Data_raw"]]))
	CKBSmRawAv$setActive(is.active = as.logical(Loaded$smoothSelection[["Data_rawAv"]]))
	CKBSmMedium$setActive(is.active = as.logical(Loaded$smoothSelection[["Data_medium"]]))
	CKBSmMediumAv$setActive(is.active = as.logical(Loaded$smoothSelection[["Data_mediumAv"]]))
	
	#### Slope
	ENTSlopeSmooth$setText(Loaded$slopeParam[3])
	ENTSlopePoints$setText(Loaded$slopeParam[2])
	CKBSlopeRaw$setActive(is.active = Loaded$slopeSelection[["Data_raw"]])
	CKBSlopeRawAv$setActive(is.active = Loaded$slopeSelection[["Data_rawAv"]])
	CKBSlopeMedium$setActive(is.active = Loaded$slopeSelection[["Data_medium"]])
	CKBSlopeMediumAv$setActive(is.active = Loaded$slopeSelection[["Data_mediumAv"]])
	
	#### Consumption
	ENTConsSat$setText(switch(class(Loaded$ConsParam["saturation"]), NULL = "NA", numeric = Loaded$ConsParam["saturation"]))
	ENTConsFactorsKLA$setText(switch(class(Loaded$ConsParam["kLa"]), NULL = "NA", numeric = Loaded$ConsParam["kLa"]))
	ENTConsFactorsKLAError$setText(switch(class(Loaded$ConsParam["kLaerror"]), NULL = "NA", numeric = Loaded$ConsParam["kLaerror"]))
	ENTConsFactorsUCon$setText(switch(class(Loaded$ConsParam["conversion"]), NULL = "NA", numeric = Loaded$ConsParam["conversion"]))
	ENTConsAxis$setText(switch(class(Loaded$ConsCharacters["axis"]), NULL = "missing", character = Loaded$ConsCharacters["axis"]))
	CKBConsLoadCal$setActive(is.active = as.logical(Loaded$ConsParam["load"]))
	ENTConsLoadCal$setText(switch(class(Loaded$ConsCharacters["file"]), NULL = "missing", character = Loaded$ConsCharacters["file"]))
	gSignalEmit(ENTConsLoadCal, "activate")
	if (length(ListKLAValueChoices) != 0) {
		if (length(Loaded$ConsCharacters["calibration"]) != 0) {
			for (i in 1:length(ListKLAValueChoices)) {
				if (class(ListKLAValueChoices[[i]])[1] == "GtkRadioButton") {
					if (ListKLAValueChoices[[i]]$getLabel() == Loaded$ConsCharacters["calibration"]) {
						ListKLAValueChoices[[i]]$setActive(is.active = TRUE)
					}
				}
			}
		}
	}
		
	CKBConsRaw$setActive(is.active = Loaded$ConsSelection[["Data_raw"]])
	CKBConsRawAv$setActive(is.active = Loaded$ConsSelection[["Data_rawAv"]])
	CKBConsMedium$setActive(is.active = Loaded$ConsSelection[["Data_medium"]])
	CKBConsMediumAv$setActive(is.active = Loaded$ConsSelection[["Data_mediumAv"]])
	#CKBConsLoadCal$setActive(is.active = as.log)
	
	## IC50 Parameters
	ENTIC50TimeStart$setText(switch(class(Loaded$IC50Time["time_start"]), NULL = "NA", numeric = Loaded$IC50Time["time_start"]))
	ENTIC50TimeEnd$setText(switch(class(Loaded$IC50Time["time_end"]), NULL = "NA", numeric = Loaded$IC50Time["time_end"]))
	ENTIC50Skip$setText(switch(class(Loaded$IC50Time["skip"]), NULL = "NA", numeric = Loaded$IC50Time["skip"]))
	ENTIC50Axis$setText(switch(class(Loaded$IC50Axis), NULL = "missing", character = Loaded$IC50Axis))
	if (length(Loaded$IC50model) != 0) {
		if (Loaded$IC50model == "LL.2") COMIC50Model$setActive(0)
		else if (Loaded$IC50model == "LL.3") COMIC50Model$setActive(1)
		else if (Loaded$IC50model == "LL.4") COMIC50Model$setActive(2)
		else if (Loaded$IC50model == "LL.5") COMIC50Model$setActive(3)
	}
	if (length(Template$Name) != 0 & length(Loaded$IC50doses$Dose) != 0) {
		for (i in 1:length(Template$Name)) {
			gtkEntrySetText(TEMPDoses[[Template$Name[i]]]$entry, Loaded$IC50doses$Dose[which(Loaded$IC50doses$Name == Template$Name[i])])
		}
	}
	
	#### Activate all elements again to set Settings
	loading <<- FALSE
	FCheckAnalysisOpt(CheckInputAnalysisOpt)

	#### IC50 Selection
	if (length(Loaded$IC50) != 0) {
		if (length(which(names(ListDataSets) == Loaded$IC50)) != 0) {
			gSignalHandlerBlock(COMIC50Selection, SelectionHandler)
			for (i in 0:8) {
				COMIC50Selection$setActive(i)
				if (length(COMIC50Selection$getActiveText()) != 0) {
					if (COMIC50Selection$getActiveText() == ListDataSets[which(names(ListDataSets) == Loaded$IC50)]) {
						gSignalHandlerUnblock(COMIC50Selection, SelectionHandler)
						COMIC50Selection$setActive(i)
						break
					
					}
				}
			}
			gSignalEmit(COMIC50Selection, "changed")
		}
	}
  
	#### Formatting Selection
	if (class(Loaded$Formatting) == "character") {
		if (length(which(names(ListDataSets) == Loaded$Formatting)) != 0) {
			gSignalHandlerBlock(COMFormatSelectData, SelectionHandlerFormatting)
			unblocked <- FALSE
			for (i in 0:8) {
				COMFormatSelectData$setActive(i)
				if (length(COMFormatSelectData$getActiveText()) != 0) {
					if (COMFormatSelectData$getActiveText() == ListDataSets[which(names(ListDataSets) == Loaded$Formatting)]) {
						gSignalHandlerUnblock(COMFormatSelectData, SelectionHandlerFormatting)
						COMFormatSelectData$setActive(i)
						unblocked <- TRUE
						break
					
					}
				}
			}
			if (unblocked == FALSE) gSignalHandlerUnblock(COMFormatSelectData, SelectionHandlerFormatting)
			gSignalEmit(COMFormatSelectData, "changed")
		} else {
			COMFormatSelectData$setActive(0)
		}
	}
}

####
#### Function: Setting -> Widgetfunction Graph Output
#####################################################
FTranslateSettingsGraphOutput <- function(Loaded) {
	if (length(Loaded$Scalebar) == 5) SETGraphOutput$Scalebar <<- Loaded$Scalebar
	gSignalEmit(COMScalebar, "changed")
	if (length(Loaded$IC50) != 0) {
		if (Loaded$IC50["type"] == "average") btnCurrentGraphIC50TypeAv$setActive(is.active = TRUE)
		else if (Loaded$IC50["type"] == "obs") btnCurrentGraphIC50TypeObs$setActive(is.active = TRUE)
		else if (Loaded$IC50["type"] == "all") btnCurrentGraphIC50TypeAll$setActive(is.active = TRUE)
		else if (Loaded$IC50["type"] == "none") btnCurrentGraphIC50TypeNone$setActive(is.active = TRUE)
		else if (Loaded$IC50["type"] == "bars") btnCurrentGraphIC50TypeBars$setActive(is.active = TRUE)
		if (Loaded$IC50["diff"] == "colour") btnCurrentGraphIC50DiffCol$setActive(is.active = TRUE)
		else if (Loaded$IC50["diff"] == "lty") btnCurrentGraphIC50DiffLty$setActive(is.active = TRUE)
		else if (Loaded$IC50["diff"] == "none") btnCurrentGraphIC50DiffNone$setActive(is.active = TRUE)
	}
}

####
#### Function: Setting -> Widgetfunction Data Output
####################################################
FTranslateSettingsDataOutput <- function(Loaded) {
	#### separators
	ENTOutputSeparator$setText(switch(class(Loaded$separator), NULL = "NA", character = Loaded$separator))
	ENTOutputDecimal$setText(switch(class(Loaded$decimal), NULL = "NA", character = Loaded$decimal))
	
	SETDataOutput$folder <<- Loaded$folder
	
	if (length(names(Loaded$export)) != 0 & length(names(Loaded$filenames)) != 0) {
		#### export files
		for (i in 1:length(Loaded$export)) {
				if (class(ListOutputLayouts[[names(Loaded$export[i])]][["ckb"]])[1] == "GtkCheckButton") ListOutputLayouts[[names(Loaded$export[i])]][["ckb"]]$setActive(is.active = as.logical(Loaded$export[i]))
		}
		
		#### filenames
		for (i in 1:length(Loaded$export)) {
			if (class(ListOutputLayouts[[names(Loaded$export[i])]][["entry"]])[1] == "GtkEntry") ListOutputLayouts[[names(Loaded$export[i])]][["entry"]]$setText(Loaded$filenames[names(Loaded$export[i])])
		}
	} else if (length(Loaded$export) != 0) {
		for (i in 1:length(Loaded$export)) {
			if (class(ListOutputLayouts[[names(Loaded$export[i])]][["ckb"]])[1] == "GtkCheckButton") ListOutputLayouts[[i]][["ckb"]]$setActive(is.active = FALSE)
			if (class(ListOutputLayouts[[names(Loaded$export[i])]][["entry"]])[1] == "GtkEntry") ListOutputLayouts[[i]][["entry"]]$setText("missing")
		}
	}
	#### Activate all elements for Data Output
	loading <<- FALSE
	FCheckDataOutput(CheckInputDataOutput)
}

#########################################################################################
#### Import / Export Settings popup
#########################################################################################
gSignalConnect(btnSettings, "clicked", after = TRUE, quote({
	POPSaveSettings <- gtkMessageDialog(parent = mainApplication, "destroy-with-parent", type = "question", buttons = "none", paste("Please enter the path/filename (.txt), starting in your current working directory, or choose a file to be loaded.\n\n", getwd(), sep=""))
	POPSaveSettings$addButton("Save", 1)
	POPSaveSettings$addButton("Load", 2)
	IMDirectorySettings <- gtkImageNewFromPixbuf(PBDirectory[[1]])
	POPSaveSettings[["actionArea"]]$getChildren()[[1]]$setImage(IMDirectorySettings)
	POPSaveSettings$addButton("Cancel", 3)
	ENTSettingsFile <- gtkEntry()
	ENTSettingsFile$setSizeRequest(-1, 20)
	ERRSettingsLoad <- gtkLabel("No such file found!")
	ERRSettingsLoad["visible"] <- FALSE
	POPSaveSettings[["vbox"]]$add(ERRSettingsLoad)
	POPSaveSettings[["vbox"]]$add(ENTSettingsFile)
	initiation <- 1
	while (initiation == 1) {
		response <- POPSaveSettings$run()[1]
		if (response == 3){
			initiation <- 0
			POPSaveSettings$destroy()
		} else if (response == 1) {
			initiation <- 0
			SettingsFile <- ENTSettingsFile$getText()
			FSaveSettings(SettingsFile)
		} else if (response == 2) {
			initiation <- 0
			SettingsFile <- ENTSettingsFile$getText()
			if (SettingsFile == "") {
				FFileChooserOpen(ENTSettingsFile)
				SettingsFile <- ENTSettingsFile$getText()
			}
			if (file.exists(SettingsFile) == FALSE) {
				ERRSettingsLoad["visible"] <- TRUE
				initiation <- 1
			} else {
				ERRSettingsLoad["visible"] <- FALSE
				if (length(try(FLoadSettings(SettingsFile))) != 0) {
					POPSaveSettingsFail <- gtkMessageDialog(parent=mainApplication, "destroy-with-parent", "error", buttons = "ok", "The settings could not be fully loaded!\nPlease check the console to see which settings are affected.")
					POPSaveSettingsFail$run()
					POPSaveSettingsFail$destroy()
				} else {
					POPSaveSettingsFail <- gtkMessageDialog(parent=mainApplication, "destroy-with-parent", "info", buttons = "ok", "The settings were successfully loaded.")
					POPSaveSettingsFail$run()
					POPSaveSettingsFail$destroy()
				}
				if (class(POPSaveSettings)[1] == "GtkMessageDialog") POPSaveSettings$destroy()
			}
		} else {
			initiation <- 0
			POPSaveSettings$destroy()
		}
	}
}))

#########################################################################################
#### Import / Export RData popup
#########################################################################################

gSignalConnect(btnRData, "clicked", after = TRUE, quote({
	POPSaveRData <- gtkMessageDialog(parent = mainApplication, "destroy-with-parent", type = "question", button = "none", paste("Please enter the path/filename (.RData), starting in your current working directory, or choose a file to be loaded.\n\n", getwd(), sep=""))
	POPSaveRData$addButton("Save", 1)
	POPSaveRData$addButton("Load", 2)
	IMDirectoryRData <- gtkImageNewFromPixbuf(PBDirectory[[1]])
	POPSaveRData[["actionArea"]]$getChildren()[[1]]$setImage(IMDirectoryRData)
	POPSaveRData$addButton("Cancel", 3)
	ENTRDataFile <- gtkEntry()
	ENTRDataFile$setSizeRequest(-1, 20)
	ERRRDataLoad <- gtkLabel("No such file found!")
	ERRRDataLoad["visible"] <- FALSE
	POPSaveRData[["vbox"]]$add(ERRRDataLoad)
	POPSaveRData[["vbox"]]$add(ENTRDataFile)
	initiation <- 1
	while (initiation == 1) {
		response <- POPSaveRData$run()
		if (response == 3) {
			initiation <- 0
			POPSaveRData$destroy()
		} else if (response == 1) {
			initiation <- 0
			RDataFile <- ENTRDataFile$getText()
			FSaveRData(RDataFile)
		} else if (response == 2) {
			initiation <- 0
			RDataFile <- ENTRDataFile$getText()
			if (RDataFile == "") {
				FFileChooserOpen(ENTRDataFile)
				RDataFile <- ENTRDataFile$getText()
			}
			if (file.exists(RDataFile) == FALSE) {
				ERRRDataLoad["visible"] <- TRUE
				initiation <- 1
			} else {
				ERRRDataLoad["visible"] <- FALSE
				if (class(try({
					load(RDataFile)
					FTemplateTableRestore(Template)
					})) == "try-error") {
					POPSaveRDataFail <- gtkMessageDialog(parent=mainApplication, "destroy-with-parent", "error", buttons = "ok", "The RData-file could not be fully loaded!\nPlease check the console for errors.")
					POPSaveRDataFail$run()
					POPSaveRDataFail$destroy()
				} else {
					POPSaveRDataFail <- gtkMessageDialog(parent=mainApplication, "destroy-with-parent", "info", buttons = "ok", "The RData-file was successfully loaded.")
					POPSaveRDataFail$run()
					POPSaveRDataFail$destroy()
				}
				if (firstAnalysis == TRUE) source("Program/Analyser_4.R")
				if (class(POPSaveRData)[1] == "GtkMessageDialog") POPSaveRData$destroy()
			}
		} else {
			initiation <- 0
			POPSaveRData$destroy()
		}
	}
}))