#########################################################################################
#### Acquired Settings
#########################################################################################
## The settings acquired in this section are saved in SETLabelNumberColour and are:
## - templatePath: direction to load / save the template
## - tempalteSep: cell separator for the template csv
## - Well: fixed setting holding the required headlines of the template columns.
SETLabelNumberColour <- list(	templatePath = "default_template.csv",
								templateSep = ";",
								Well = c("X", "Name", "Number", "Colour", "Linetype") )

#########################################################################################
#### Layout and Interface Definition for Labels & Colours
#########################################################################################

####
#### Frame layout
#################
## The screen is horizontally separated in two frames. The upper frame contains the
## preview of the template and the lower one contains the interface for saving and
## loading it.

FRTemplateTable <- gtkFrame()
FRTemplateSvLd <- gtkFrame("Save or load labels")

LBLLabelsColoursFrame <- gtkLabel("Please fill in the template for each well. This will determine the graph design.")
LBLLabelsColoursFrame$setSizeRequest(-1, 25)

layoutLabels <- gtkVBox(FALSE, 5)
	layoutLabels$packStart(LBLLabelsColoursFrame, expand = FALSE, fill = FALSE)
	layoutLabels$packStart(FRTemplateTable, expand = TRUE, fill = TRUE)
	layoutLabels$packStart(FRTemplateSvLd, expand = FALSE, fill = FALSE)

####
#### Template Preview Frame
###########################
## In this frame the instructions for the template are given:
#### "If you want a well to be excluded completely name it 'Exclude'.
## It will also be removed from the raw data set!"
#### In the centre of the frame the template is displayed in a scrollable
## view port. It is created by the function FTemplateTableRestore (see section
## Labels & Colours Execute). Here only the renderers for each column besides the
## first are defined.
####In order to make filling the template easier
## several auto fill buttons are added underneath the template:
## - "Autofill labels": creates blank template containing the well labels from the input data
## - "Autofill numbers": fills the "Number" column with 1 : well number
## - "All black": sets all colours to black
## - "All solid": sets all line types to solid
BXTemplateTable <- gtkVBox(FALSE, 5)
BXTemplateTable$setBorderWidth(5)
FRTemplateTable$setBorderWidth(3)
	FRTemplateTable$add(BXTemplateTable)

BXTemplateTableInstructions <- gtkHBox(FALSE, 5)
BXTemplateTableButtonsPack <- gtkHBox(FALSE, 5)
	BXTemplateTable$packStart(BXTemplateTableInstructions, expand = FALSE, fill = FALSE)
	BXTemplateTable$packEnd(BXTemplateTableButtonsPack, expand = FALSE, fill = FALSE)

# Instructions
LBLTemplateTableInstructions <- gtkLabel("If you want a well to be excluded completely name it 'Exclude'. It will also be removed from the raw data set!")
	BXTemplateTableInstructions$packStart(LBLTemplateTableInstructions, expand = FALSE, fill = FALSE)

# Template Table renderers
renderer1 <- gtkCellRendererTextNew()
renderer1["editable"] <- TRUE
renderer2 <- gtkCellRendererTextNew()
renderer2["editable"] <- TRUE
renderer3 <- gtkCellRendererTextNew()
renderer3["editable"] <- TRUE
renderer4 <- gtkCellRendererTextNew()
renderer4["editable"] <- TRUE

# Buttons
BXTemplateTableButtons <- gtkHButtonBox(TRUE)
	BXTemplateTableButtonsPack$packStart(BXTemplateTableButtons, expand = FALSE, fill = FALSE)
btnAutofillWells <- gtkButton("Autofill labels")
btnAutofillNumbers <- gtkButton("Autofill numbers")
btnAutofillColour <- gtkButton("All black")
btnAutofillLinetype <- gtkButton("All solid")
	BXTemplateTableButtons$packStart(btnAutofillWells, expand = FALSE, fill = FALSE)
	BXTemplateTableButtons$packStart(btnAutofillNumbers, expand = FALSE, fill = FALSE)
	BXTemplateTableButtons$packStart(btnAutofillColour, expand = FALSE, fill = FALSE)
	BXTemplateTableButtons$packStart(btnAutofillLinetype, expand = FALSE, fill = FALSE)

####
#### Save / Load Frame
######################
## In this frame only two buttons for saving and loading and
## an entrybox for the csv separator are present. The two buttons both open a popup.
## The entrybox for the file path is also defined here, but it is not visible to the
## user and only used for storing the path before it's actually saved as setting.
BXTemplateSvLdV <- gtkVBox(FALSE, 5)
BXTemplateSvLdV$setBorderWidth(5)
FRTemplateSvLd$setBorderWidth(3)
	FRTemplateSvLd$add(BXTemplateSvLdV)

BXTemplateInstructions <- gtkVBox(FALSE, 5)
BXTemplateSvLdH1 <- gtkHBox(FALSE, 5)
	BXTemplateSvLdV$packStart(BXTemplateInstructions, expand = FALSE, fill = FALSE)
	BXTemplateSvLdV$packStart(BXTemplateSvLdH1, expand = FALSE, fill = FALSE)

# Template Instructions
LBLTemplateInstructions <- gtkLabel("The required format of the template is a .txt or .csv file with a title for every column but the first.")
	BXTemplateInstructions$packStart(LBLTemplateInstructions, expand = FALSE, fill = FALSE)

# Template H1
BXTemplateSvLdButtons <- gtkHButtonBox(TRUE)
	BXTemplateSvLdH1$packStart(BXTemplateSvLdButtons, expand = FALSE, fill = FALSE)

btnTemplateSave <- gtkButton("Save template")
btnTemplateLoad <- gtkButton("Load template")
IMDirectoryTemplate <- gtkImageNewFromPixbuf(PBDirectory[[1]])
btnTemplateLoad$setImage(IMDirectoryTemplate)
	BXTemplateSvLdButtons$packStart(btnTemplateSave, expand = FALSE, fill = FALSE)
	BXTemplateSvLdButtons$packStart(btnTemplateLoad, expand = FALSE, fill = FALSE)
ENTTemplatePath <- gtkEntry() # Invisible just for storing the documentpath
ENTTemplateSeparator <- gtkEntry()
ENTTemplateSeparator$setSizeRequest(30, 20)
LBLTemplateSeparator <- gtkLabel("Separator:")
	BXTemplateSvLdH1$packStart(LBLTemplateSeparator, expand = FALSE, fill = FALSE)
	BXTemplateSvLdH1$packStart(ENTTemplateSeparator, expand = FALSE, fill = FALSE)

#########################################################################################
#### Labels & Colours Interactive
#########################################################################################

####
#### Signals: Column renderers
##############################
## The column renderers each execute a function when a field in the column is edited.
## This function uses cell, path.string and new.text, which are all provided by the
## signal itself. First thing is to extract the line from path.string. It is
## therefore converted to a tree path from which the indices can be read. As there is
## only one node in our tree the only index is our line - 1 (zero based counting).
#### Using this index new.text can be saved to the template data frame and the
## template is then checked and restored.
gSignalConnect(renderer1, "edited", function(cell, path.string, new.text) {
	index <- gtkTreePathNewFromString(path.string)$getIndices() +1
	Template[index, 2] <<- new.text
	FCheckTemplateLoaded(Template)
	if (Template[1,1] != "empty") {
		FTemplateTableRestore(Template)
	} else {
		gSignalEmit(btnAutofillWells, "clicked")
	}
})
gSignalConnect(renderer2, "edited", function(cell, path.string, new.text) {
	index <- gtkTreePathNewFromString(path.string)$getIndices() +1
	Template[index, 3] <<- new.text
	FCheckTemplateLoaded(Template)
	if (Template[1,1] != "empty") {
		FTemplateTableRestore(Template)
	} else {
		gSignalEmit(btnAutofillWells, "clicked")
	}
})

gSignalConnect(renderer3, "edited", function(cell, path.string, new.text) {
	index <- gtkTreePathNewFromString(path.string)$getIndices() +1
	Template[index, 4] <<- new.text
	FCheckTemplateLoaded(Template)
	if (Template[1,1] != "empty") {
		FTemplateTableRestore(Template)
	} else {
		gSignalEmit(btnAutofillWells, "clicked")
	}
})
gSignalConnect(renderer4, "edited", function(cell, path.string, new.text) {
	index <- gtkTreePathNewFromString(path.string)$getIndices() +1
	Template[index, 5] <<- new.text
	FCheckTemplateLoaded(Template)
	if (Template[1,1] != "empty") {
		FTemplateTableRestore(Template)
	} else {
		gSignalEmit(btnAutofillWells, "clicked")
	}
})
####
#### Signal: Entry for Template Separator and File Name
#######################################################
## These signal handlers only retrieve the plain text from the entryboxes and saves it to
## SETLabelNumberColour$templateSep and -$templatePath respectively. The latter one is
## only called by the signal handler of the loading button. In case the filechooser
## doesn't work properly and the file can't be found the error message is shown.
#### Error message "-- Entered path doesn't lead to an existing file! --"
gSignalConnect(ENTTemplateSeparator, "activate", quote({
	text <- ENTTemplateSeparator$getText()
	SETLabelNumberColour$templateSep <- text
}))

ERRTemplatePath <- gtkLabel("-- Entered path doesn't lead to an existing file! --")
	ERRTemplatePath["visible"] <- FALSE
	BXErrormessages$packStart(ERRTemplatePath, expand = TRUE, fill = TRUE)
	
gSignalConnect(ENTTemplatePath, "activate", quote({
	text <- ENTTemplatePath$getText()
	SETLabelNumberColour$templatePath <- text
}))

####
#### Signal: Autofill Labels
############################
## If the data input is still empty a popup will appear which tells the user
## to first import the data. Given that the data was already imported the
## first thing is to compare the number of columns in the data minus one with the number
## of lines in the currently loaded template. If they are equal the labels in the
## template are replaced with the column names of the data. If they differ in length
## a new template of the required length is generated with the new labels. The
## colour column is automatically filled with "black". Either way the function
## FCheckTemplateLoaded is called and if it returns TRUE the template preview is restored.
gSignalConnect(btnAutofillWells, "clicked", quote({
	if (Data_input[1,1] == "empty") {
		POPDataInput <- gtkMessageDialog(mainApplication, "destroy-with-parent", "error", "ok", "Please import data first!\nOtherwise no template can be generated.")
		if (POPDataInput$run() == GtkResponseType["ok"]) POPDataInput$destroy()
	} else {
		if (length(Template$X) == length(names(Data_input))-1) {
			Template$X <- names(Data_input)[2:dim(Data_input)[2]]
			if (FCheckTemplateLoaded(Template) == TRUE) {
				FTemplateTableRestore(Template)
			}
		} else {
			mat <- matrix(0, nrow = (dim(Data_input)[2]-1), ncol = 5)
			Template <- as.data.frame(mat)
			names(Template) <- SETLabelNumberColour$Well
			Template$X <- names(Data_input)[2:dim(Data_input)[2]]
			Template$Colour <- rep("black", length.out = dim(Data_input)[2]-1)
			if (FCheckTemplateLoaded(Template) == TRUE) {
				FTemplateTableRestore(Template)
			}
		}
	}
}))

####
#### Signal: Autofill Numbers
#############################
## When the autofill numbers button is clicked the column is
## automatically filled with numbers from one to x where x is the
## number of rows in the template. Afterwards the template is checked
## for validity and restored.
gSignalConnect(btnAutofillNumbers, "clicked", quote({
	Template$Number <- c(1:length(Template$X))
	FCheckTemplateLoaded(Template)
	if (Template[1,1] != "empty") {
		FTemplateTableRestore(Template)
	} else {
		gSignalEmit(btnAutofillWells, "clicked")
	}
}))

####
#### Signal: All black / solid
##############################
## When one of these buttons is clicked, the respective column is
## filled with a replicate of either "black" or "solid". Afterwards the template
## is checked for validity and restored.
gSignalConnect(btnAutofillColour, "clicked", quote({
	Template$Colour <- c(rep("black", length(Template$X)))
	FCheckTemplateLoaded(Template)
	if (Template[1,1] != "empty") {
		FTemplateTableRestore(Template)
	} else {
		gSignalEmit(btnAutofillWells, "clicked")
	}
}))
gSignalConnect(btnAutofillLinetype, "clicked", quote({
	Template$Linetype <- c(rep("solid", length(Template$X)))
	FCheckTemplateLoaded(Template)
	if (Template[1,1] != "empty") {
		FTemplateTableRestore(Template)
	} else {
		gSignalEmit(btnAutofillWells, "clicked")
	}
}))

####
#### Signal: Save button clicked:
#################################
## For saving the template two functions are called. First comes
## FCheckLabelColour to make sure that the separator setting is acquired.
## Second comes FSaveTemplate.
gSignalConnect(btnTemplateSave, "clicked", quote({
	FCheckLabelColour(CheckInputLabelColour)
	FSaveTemplate(SETLabelNumberColour$templatePath)
}))

####
#### Signal: Load button clicked:
#################################
## When loading a template first FCheckLabelColour is called and next a filechooser
## is opened with the hidden entrybox for the template destination as target
## (FFileChooserOpen see section mainApplication - global functions). Then the signal
## for this hidden entrybox is emitted in order to save the destination. If the file
## the corresponding error message is shown, otherwise hidden. If it's valid the current
## template is emptied and the function FLoadTemplate is called taking its parameters
## from the settings.
gSignalConnect(btnTemplateLoad, "clicked", quote({
	FCheckLabelColour(CheckInputLabelColour)
	FFileChooserOpen(ENTTemplatePath)
	gSignalEmit(ENTTemplatePath, "activate")
	if (file.exists(SETLabelNumberColour$templatePath) == TRUE) {
		ERRTemplatePath["visible"] <- FALSE
		Template <- data.frame() #delete old template
		FLoadTemplate(SETLabelNumberColour$templatePath, SETLabelNumberColour$templateSep)
	} else {
		ERRTemplatePath["visible"] <- TRUE
	}
}))

#########################################################################################
#### Labels & Colours Execute
#########################################################################################
## The template is first defined at this point as data frame containing "empty" = "empty"
## just as the Data_input was previously defined. Whenever the Template is restored that
## is to this initial state.
Template <- data.frame( "empty" = "empty" )

# Create vector containing all elements to check again before execution
CheckInputLabelColour <- c(ENTTemplatePath, ENTTemplateSeparator)

################################
#### Function: FCheckLabelColour
################################
## This function triggers the signal handlers of all input widgets on the screen.
## It works with a previously defined vector containing all widget names as "Input",
## detects their class and emits the corresponding signal.
FCheckLabelColour <- function(Input) {
	for (i in 1:length(Input)) {
		if (class(Input[[i]])[1] == "GtkRadioButton") {
			gSignalEmit(Input[[i]], "toggled")
		} else if (class(Input[[i]])[1] == "GtkEntry") {
			gSignalEmit(Input[[i]], "activate")
		}
	}
}

###################################
#### Function: FCheckTemplateLoaded
###################################
## This function is used whenever the validity of the currently / newly loaded
## template has to be confirmed. Its only argument is the template itself.
FCheckTemplateLoaded <- function(T) {
#### The first and most important check is for the column names.
## They have to exactly the same as in SETLabelNumberColour$Well or
## the analysis and data display won't work any longer. If one of them differs
## a popup is created, the template is restored and the function returns FALSE.
	for(i in 1:length(SETLabelNumberColour$Well)) {
		if (length(names(T)) != length(SETLabelNumberColour$Well) | names(T)[i] != SETLabelNumberColour$Well[i]) {
			POPTemplateFail <- gtkMessageDialog(mainApplication, "destroy-with-parent", "error", "ok", "The template could not be loaded, because its column titles don't fit the requirements!")
			if (POPTemplateFail$run() == GtkResponseType["ok"]) POPTemplateFail$destroy()
			Template <<- data.frame( "empty" = "empty" )
			return(FALSE)
		}
	}
	
#### If the Layout of the template in general fits the colours are validated next.
## They have to be present in the full R colors palette. If one of the given colours
## can't be found a popup is created and the function
## returns FALSE. One might want to implement colour chooser widgets on each line
## of the template preview in order to spare the user looking up the valid colors.
	for (i in 1:dim(Template)[1]) {
		if (length(which(colors() == Template$Colour[i])) == 0) {
			POPTemplateFail <- gtkMessageDialog(mainApplication, "destroy-with-parent", "error", "ok", paste("The loaded template can not be used, because ", Template$Colour[i], " is not a supported colour! Print all valid ones by typing 'colors()' in the console.", sep= ""))
			if (POPTemplateFail$run() == GtkResponseType["ok"]) POPTemplateFail$destroy()
			return(FALSE)
		}
	}
	
#### In order to check whether the names for the wells match between the template
## and the data input both vectors are cast as factor to sort them. Then they are
## one after another. If one doesn't match a popup is created and the function
## returns FALSe, but the template is not reset. This way the user can first load a
## new template and after that the matching data.
	if (Data_input[1,1] != "empty") {
		wellnamesData <- as.factor(names(Data_input)[2:dim(Data_input)[2]])
		wellnamesTemplate <- as.factor(T$X)
		for (i in 1:length(wellnamesData)) {
			if (levels(wellnamesData)[i] != levels(wellnamesTemplate)[i]) {
				POPTemplateFail <- gtkMessageDialog(mainApplication, "destroy-with-parent", "error", "ok", "Well numbers in the template and imported data don't match!")
				if (POPTemplateFail$run() == GtkResponseType["ok"]) POPTemplateFail$destroy()
				return(FALSE)
			}
		}
	}
	
#### Last but not least the template is screened for empty entries. If there is
## at least one the user is asked whether this is on purpose or not. If it is
## the function returns TRUE and if it's not FALSE is returned.
	for (i in 2:dim(T)[2]) {
		for (j in 1:dim(T)[1]) {
			if (T[j,i] == "") {
				POPTemplateFail <- gtkMessageDialog(mainApplication, "destroy-with-parent", "question", "none", "There are empty fields in the template. Are you sure you want to proceed?")
				POPTemplateFail$addButton("Yes", 1)
				POPTemplateFail$addButton("No", 2)
				if (POPTemplateFail$run() == 1) {
					POPTemplateFail$destroy()
					return(TRUE)
				} else {
					POPTemplateFail$destroy()
					return(FALSE)
				}
			}
		}
	}
	
#### If the function hasn't gone through any exception until this point it returns
## TRUE.
	return(TRUE)
}

####
#### Function: FSaveTemplate
############################
## This function creates a message dialogue with buttons "save" and "cancel" and
## an entrybox for the file name. If the user clicks "save" and FCheckFileExistance
## returns TRUE (see section mainApplication - global functions) the template is
## saved with the given settings and a short message is printed to the R console.
## In any other case the dialogue is destroyed without any further action.
FSaveTemplate <- function(path) {
	POPExportTemplate <- gtkMessageDialog(mainApplication, "destroy-with-parent", type = "question", buttons = "none", paste("Please enter a path/filename (.txt or .csv), starting in your current working directory:\n\n", getwd(), "/...", sep=""))
	POPExportTemplate$addButton("Save", 1)
	POPExportTemplate$addButton("Cancel", 2)
	ENTTemplateOutput <- gtkEntry()
	ENTTemplateOutput$setSizeRequest(-1, 20)
	ENTTemplateOutput$setText("Template.csv")
		POPExportTemplate[["vbox"]]$add(ENTTemplateOutput)
	if (POPExportTemplate$run() == 1 && FCheckFileExistance(ENTTemplateOutput$getText()) == TRUE) {
		output <- file(paste(getwd(), ENTTemplateOutput$getText(), sep="/"), open ="w")
		write.table(Template, file = output, sep = SETLabelNumberColour$templateSep, dec = SETDataOutput$decimal, col.names = TRUE, row.names = FALSE)
		close(output)
		print(paste("Template was saved to ", paste(getwd(), ENTTemplateOutput$getText(), sep="/"), sep=""))
		POPExportTemplate$destroy()
	} else POPExportTemplate$destroy()
}

####
#### Function: FLoadTemplate
############################
## This function reads the table using the path and separator as input
## parameters. The name of the first column is replaced with X and if the
## template is valid it is shown on screen. Afterwards the entryboxes for
## the IC50 doses in the analysis options are refreshed using the new or empty
## template.
FLoadTemplate <- function(path, separator) {
	Template <<- read.table(path, header = TRUE, sep = separator, colClasses = "character")
	names(Template)[1] <- "X"
	FCheckTemplateLoaded(Template)
	if (Template[1,1] != "empty") {
		FTemplateTableRestore(Template)
	} else {
		gSignalEmit(btnAutofillWells, "clicked")
	}
	FRefreshIC50Doses(Template)
}

####
#### Function: FTemplateTableRestore
####################################
## This function is needed to display the template data frame in the user interface.
## Its only argument is the template. First of all the currently displayed scrolled
## window is destroyed. The renderers for the different columns remain unaffected.
## Then the template is first converted to a rGtkDataFrame and then further on to
## a gtkTreeView. Then the columns to display are added to the treeview with
## the different column names and renderers as parameters. Then the scrolled window is
## recreated and the treeview is added. Finally the IC50 doses on the Analysis Options
## screen are refreshed by a call to FRefreshIC50Doses.
FTemplateTableRestore <- function(Temp) {
	if (length(BXTemplateTable$getChildren()) > 2) {
		BXTemplateTable$getChildren()[[2]]$destroy()
	}
	
	TM <- rGtkDataFrame(Temp)
	TVTemplate <- gtkTreeView(TM)
	
	# render first column
	renderer0 <- gtkCellRendererTextNew()
	TVTemplate$insertColumnWithAttributes(-1, title = "Well", renderer0, text = 1-1)
	# render second column
	TVTemplate$insertColumnWithAttributes(-1, title = "Name", renderer1, text = 2-1)
	# render third column
	TVTemplate$insertColumnWithAttributes(-1, title = "Number", renderer2, text = 3-1)
	# render fourth column
	TVTemplate$insertColumnWithAttributes(-1, title = "Colour", renderer3, text = 4-1)
	# render fifth column
	TVTemplate$insertColumnWithAttributes(-1, title = "Linetype", renderer4, text = 5-1)
	
	SCRTemplateTable <- gtkScrolledWindow()
	SCRTemplateTable$setPolicy("automatic", "automatic")
	BXTemplateTable$packStart(SCRTemplateTable, expand = TRUE, fill = TRUE)
		
	SCRTemplateTable$add(TVTemplate)
	
	FRefreshIC50Doses(Temp)
}